;;
;; export-sheet.scm
;;
;; Export attendance sheet as MarkDown document.
;;
;; ISC License
;;
;; Copyright 2024 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit export-sheet))

(module
 export-sheet
 (
  print-attendance-sheet
  )

 (import scheme
	 (chicken base)
	 (chicken string)
	 (chicken format)
	 srfi-1
	 mbase
	 brmember
	 brmember-format
	 util-bst-ldict
	 members-payments
	 util-format
	 members-fees
	 cal-period)

 (define (print-attendance-sheet MB)
   (print "\\documentclass[10pt]{article}")
   (print "\\usepackage[top=1cm,left=2cm,right=2cm,bottom=2cm]{geometry}")
   (print "\\begin{document}")
   (print "\\begin{center}")
   (print
    (format
     "Prezenční listina ~A. Valné Hromady brmlab z.s. konané ~A v sídle spolku"
     666
     "1.2.3456"))
   (newline)
   (print "\\vskip1em")
   (newline)
   (define colnames
     '((id) Nick Name Surname (Fee) (Balance) (Active) Signature))
   (print
    (format
     "\\begin{tabular}{|~A|}"
     (string-intersperse
      (map
       (lambda (x)
	 (if (symbol? x)
	     "l" "r"))
       colnames)
      "|")))
   (print "\\hline")
   (print
    (string-intersperse
     (map
      (lambda (x)
	(format
	 "\\textbf{~A}"
	 (if (symbol? x)
	     (symbol->string x)
	     (symbol->string (car x)))))
      colnames)
     "&")
    "\\\\")
   (print "\\hline")
   (let loop ((mrs (find-members-by-predicate
		    MB (lambda (mr)
			 (brmember-active? mr)))))
     (when (not (null? mrs))
       (let* ((mr (car mrs))
	      (info (ldict-ref mr 'info))
	      (name (ldict-ref info 'name "ERROR"))
	      (name* (string-translate*
		      name
		      '(("_" . " "))))
	      (namel (string-split name*))
	      (sname (car (reverse namel)))
	      (fname
	       (string-intersperse
		(reverse
		 (cdr
		  (reverse namel)))
		" "))
	      (cal (member-calendar mr))
	      (rcal (reverse cal))
	      (rcal12
	       (if (> (length rcal) 12)
		   (take rcal 12)
		   rcal))
	      (acal12 (map cadr rcal12))
	      (acal12* (map (lambda (f) (if (memq 'active f) 1 0)) acal12))
	      (numactive (foldl + 0 acal12*))
	      (spec-fee (brmember-spec-fee mr))
	      (current-fee (if spec-fee
			       spec-fee
			       (member-calendar-entry->fee
				(list (*current-month*)
				      (brmember-flags mr)
				      spec-fee))))
	      )
	 (print
	  (brmember-id mr)
	  " & "
	  (string-translate*
	   (brmember-nick mr)
	   '(("_" . "\\_")))
	  " & "
	  fname
	  " & "
	  sname
	  " & "
	  (format-amount current-fee)
	  " & "
	  (format-amount
	   (member-total-balance mr))
	  " & "
	  numactive
	  " & "
	  "~ ~ ~ ~ ~"
	  " \\\\")
	 (print "\\hline")
	 (loop (cdr mrs)))))
   (print "\\end{tabular}")
   (print "\\end{center}")
   (print "\\end{document}"))

 )
